use rusqlite::{types::*, *};

const DBPATH: &'static str = "data/tracker.db";
const STATEMENTS: [&str; 2] = [
    "
CREATE TABLE log (
   id integer primary key autoincrement,
   time text,
   serial integer,
   latitude float,
   longitude float,
   speed integer,
   height integer,
   direction integer);",
    "
CREATE INDEX idx_time
     ON log (time);",
];
const QUERY_INSERT: &'static str = "
INSERT INTO log (
  time,
  serial,
  latitude,
  longitude,
  speed,
  height,
  direction
)
VALUES (
  :time,
  :serial,
  :latitude,
  :longitude,
  :speed,
  :height,
  :direction
)";

pub fn connectdb() -> Result<Connection> {
    let conn = Connection::open(DBPATH)?;
    Ok(conn)
}

pub fn initdb(conn: &Connection) -> Result<()> {
    create_tables(&conn)?;
    set_pragmas(&conn)?;
    Ok(())
}

fn create_tables(conn: &Connection) -> Result<()> {
    for s in STATEMENTS {
        match conn.execute(s, ()) {
            Ok(_) => {}
            Err(err) => println!("update failed: {}", err),
        }
    }
    Ok(())
}

fn set_pragmas(conn: &Connection) -> Result<()> {
    conn.pragma_update(Some(DatabaseName::Main), "journal_mode", "WAL")?;
    Ok(())
}

pub fn prepare_insert(conn: &Connection) -> Statement {
    conn.prepare(QUERY_INSERT).unwrap()
}

pub fn insert(conn: &Connection, dblog: &DbLog) -> Result<()> {
    let mut stmt = prepare_insert(&conn);
    match stmt.execute(params![
        dblog.serial,
        dblog.time,
        dblog.latitude,
        dblog.longitude,
        dblog.speed,
        dblog.height,
        dblog.direction,
    ]) {
        Ok(i) => println!("{} rows were inserted", i),
        Err(err) => println!("insert failed: {}", err),
    }
    Ok(())
}

struct DbLog {
    serial: u16,
    time: String,
    latitude: f64,
    longitude: f64,
    speed: u16,
    height: u16,
    direction: u16,
}

impl ToSql for DbLog {
    fn to_sql(&self) -> Result<ToSqlOutput<'_>> {
        Ok(self.time.to_sql().unwrap())
    }
}

impl DbLog {
    fn to_params(&self) -> &[&dyn ToSql; 7] {
        let res = params![
            self.time,
            self.serial,
            self.latitude,
            self.longitude,
            self.speed,
            self.height,
            self.direction
        ];
        res
    }
}
